@@ -222,11 +222,12 @@ class LazyCoolerRegion(GenomicRegion):
 
     @property
     def attributes(self):
-        base_attributes = self._series.index.tolist()
-        for a in ['chromosome', 'bias', 'strand', 'start', 'end', 'weight']:
-            if a not in base_attributes:
-                base_attributes.append(a)
-        return base_attributes
+        try:
+            base_attributes = self._series.index.tolist()
+        except AttributeError:
+            base_attributes = [a for a in dir(self._series) if not a.startswith('_')]
+
+        return list(set(base_attributes + ['chromosome', 'bias', 'strand', 'start', 'end', 'weight']))
 
     @property
     def chromosome(self):
@@ -463,7 +464,7 @@ class CoolerHic(RegionMatrixContainer, cooler.Cooler):
         return mappable
 
     def bias_vector(self):
-        return np.array([r.bias for r in self.regions(lazy=True)])
+        return np.array([r.bias if np.isfinite(r.bias) else 0 for r in self.regions(lazy=True)])
 
     def expected_values_and_marginals(self, selected_chromosome=None, norm=True,
                                       *args, **kwargs):
